//
//  WiFiWeatherViewController.m
//  WiFiWeather
//
//  Created by Robert Kressin on 4/2/10.
//  Copyright KS Technologies 2010. All rights reserved.
//

#import "WiFiWeatherViewController.h"
#import "NSStreamAdditions.h"

/*
 Not all NSStream methods are available on the iPhone that exist on Mac OS X.  So, I had
 to create a class to override some NSStream methods to create the socketed connection.
 */

@implementation WiFiWeatherViewController

@synthesize dataArray;

#pragma mark -
#pragma mark View Methods
- (void)viewDidLoad {
	
	// Connect to the server - assuming an ad hoc connection here
	[self connectToServerUsingStream:@"169.254.1.1" portNo:2000];   
	
	// Init the main array storing data
	self.dataArray = [NSMutableArray array];
	
    [super viewDidLoad];
}

#pragma mark -
#pragma mark Bit Streaming Methods
-(void) connectToServerUsingStream:(NSString *)urlString 
                            portNo: (uint) portNumber 
{
	
    if (![urlString isEqualToString:@""]) {
		
		// it's a valid URL
        NSURL *website = [NSURL URLWithString:urlString];
		
        if (!website) {
			
			// but I can't get to it
            NSLog(@"%@ cannot be reached.");
            return;
			
        } else {
			NSLog(@"URL is valid; attempting to create a network connection.");
            [NSStream getStreamsToHostNamed:urlString 
                                       port:portNumber 
                                inputStream:&iStream
                               outputStream:&oStream];     
			
            [iStream retain];
            [oStream retain];
            
            [iStream setDelegate:self];
            [oStream setDelegate:self];
            
            [iStream scheduleInRunLoop:[NSRunLoop currentRunLoop]
                               forMode:NSDefaultRunLoopMode];
            [oStream scheduleInRunLoop:[NSRunLoop currentRunLoop]
                               forMode:NSDefaultRunLoopMode];
            
            [oStream open];
            [iStream open];            
        }
	}    
	
}

-(void) writeToServer:(const uint8_t *) buf {
	// in this example, I'm only receiving data, but this is handy for duplex communication
    [oStream write:buf maxLength:strlen((char*)buf)];    
}

// NSStream delegate method responsible for handling byte-level communications
- (void)stream:(NSStream *)stream handleEvent:(NSStreamEvent)eventCode {
    
    switch(eventCode) {
        case NSStreamEventHasBytesAvailable:
        {
			
			NSLog(@"Data is available from the WiFi Module.");
            
			// First time using the NSMutableData object?  Initialize it.
			if (data == nil) {
                data = [[NSMutableData alloc] init];
            }
			
			// Dump the input stream into a temporary buffer
            uint8_t buf[1024];
            unsigned int len = 0;
            len = [(NSInputStream *)stream read:buf maxLength:1024];
            
			// Dump the temporary buffer into an NSData object
			if(len) {    
				
                [data appendBytes:(const void *)buf length:len];
                int bytesRead;
                bytesRead += len;
				
            } else {
                
				NSLog(@"Check your packet; it is 0-bytes long!");
				
            }
			
			// At this point, I would normally dump this data to a
			// convenience method for processing and display.  But,
			// this is just a demo.  ;-)
			
			// The first two bytes in the packet are for temperature.
			// The second two bytes in the packet are for humidity.
			NSRange temperatureRange = {0,1};
			NSRange humidityRange = {2,3};
			
			// Extract those bytes and store them locally.  Both contain 2-bytes / 16-bits of data.
			short tempTemperature;
			[data getBytes:&tempTemperature range:temperatureRange];	
			
			short tempHumdity;
			[data getBytes:&tempHumdity range:humidityRange];
			
			// You can ignore the man behind the curtain.  At the last minute, I found
			// a bug related to byte ordering.  I fix that bug here by reconstructing
			// the data words.
			uint8_t lowByteHumidity = (uint8_t)(tempHumdity >> 8);
			uint8_t highByteHumidity = (uint8_t)(tempHumdity);
			uint8_t lowByteTemperature = (uint8_t) (tempTemperature >> 8);
			uint8_t highByteTemperature = (uint8_t) (tempTemperature);
			
			// And then I reconstruct the 16-bit word representing temperature & humidity.
			short finalHumidity;
			finalHumidity = (lowByteHumidity) | (highByteHumidity << 8);
			
			short finalTemperature;
			finalTemperature = (lowByteTemperature) | (highByteTemperature << 8);
			
			// Humidity Calculation, based on equation provided in datasheet
			float c1 = -2.0468;
			float c2 = 0.0367;
			float c3 = -1.6E-6;
			float actualHumidity = c1 + c2*finalHumidity + c3*finalHumidity*finalHumidity;
			NSString *actualHumidityString = [NSString stringWithFormat:@"%2.2f RH", actualHumidity];
			
			// Temperature Calculation, based on equation provided in datasheet
			float d1 = -39.3;
			float d2 = 0.018;
			
			float actualTemperature = d1 + d2 * finalTemperature;
			NSString *actualTemperatureString = [NSString stringWithFormat:@"%2.2f degF", actualTemperature];
			
			// Timestamp
			NSString *actualTimestampString = [NSString stringWithFormat:@"%@", [NSDate new]];
			
			NSDictionary *aDataDictionary = [NSDictionary dictionaryWithObjects:[NSArray arrayWithObjects:actualTemperatureString, actualHumidityString, actualTimestampString, nil] 
																		forKeys:[NSArray arrayWithObjects:@"temperature", @"humidity", @"timestamp", nil]];
											 
			[txtView setText:[NSString stringWithFormat:@"%@ %@", actualHumidityString, actualTemperatureString]];
			
			// I always want to show latest data at the top of the table
			[dataArray insertObject:aDataDictionary atIndex:0];
			[dataTableView reloadData];
			
			// Clean up, get ready for next packet
            [data release];        
            data = nil;
			
        } break;
    }
}

-(void) disconnect {
	NSLog(@"Force disconnect has occurred.");
    [iStream close];
    [oStream close];
}

#pragma mark -
#pragma mark Table view methods
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
	// Nothing fancy, just one section
    return 1;
}


- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
	
    return [dataArray count];
}


// Customize the appearance of table view cells.
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
	// Initialize the cell if necessary
    static NSString *CellIdentifier = @"Cell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:CellIdentifier] autorelease];
    }
    
	// Display the actual data	
	cell.textLabel.text = [NSString stringWithFormat:@"%@ // %@", 
						   [[dataArray objectAtIndex:indexPath.row] valueForKey:@"temperature"], 
						   [[dataArray objectAtIndex:indexPath.row] valueForKey:@"humidity"]];
	
	cell.detailTextLabel.text = [[dataArray objectAtIndex:indexPath.row] valueForKey:@"timestamp"];
	
	// No Accessory View for the Cell
	cell.accessoryType = UITableViewCellAccessoryNone;
	
    return cell;
}

/*
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
	return nil;
}
 */

#pragma mark -
#pragma mark Cleanup Methods
- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}

- (void)dealloc {
	
    [self disconnect];
	
    [iStream release];
    [oStream release];
	
    [super dealloc];
}

@end
